function v = calc_value_house(begin_time, t, hpi, nhp)
% 
%#
%# # License
%#
%# =============================================================================
%#
%# Copyright 2011, Dimitrios Bisias, Andrew W. Lo, and Stavros Valavanis
%#
%# COPYRIGHT STATUS:  This work was funded in whole or in part by the Office of 
%# Financial Research under U.S. Government contract TOSOFR-11-C-0001, and is, 
%# therefore, subject to the following license: The Government is granted for 
%# itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
%# worldwide license to reproduce, prepare derivative works,
%# distribute copies to the public, perform and display the work.  
%# All other rights are reserved by the copyright owner.
%# 
%# THIS SOFTWARE IS PROVIDED "AS IS".  YOU ARE USING THIS SOFTWARE AT YOUR OWN
%# RISK.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
%# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS, OR
%# THE UNITED STATES GOVERNMENT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
%# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
%# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
%# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%# 
%# =============================================================================
%#
%# # Synopsis
%#			
%# calc_value_house - Calculates the value of a house at time t with t beginning 
%# at begin_time. 
%#
%# # Introduction 
%# NOTE: PART OF A SET OF 8 RELATED FILES:
%#
%#  * housing_refinance.m (main function)
%#  * loan_to_value.m
%#  * balance_loan.m
%#  * prob_refinance.m
%#  * prob_survival.m
%#  * calc_value_house.m
%#  * getOptionPrice.m
%#  * dateFromIndex.m
%#
%# Khandani, Lo, and Merton (2009) posit that rising home prices, declining 
%# interest rates, and near-frictionless refinancing opportunities led to vastly
%# increased systemic risk in the financial system. A simultaneous occurrence of 
%# these factors imposes an unintentional synchronization of homeowner leverage.
%# This synchronization, coupled with the indivisibility of residential real 
%# estate that prevents homeowners from deleveraging when property values 
%# decline and homeowner equity deteriorates, conspire to create a ratchet 
%# effect in which homeowner leverage is maintained or increased during good 
%# times without the ability to decrease leverage during bad times. To measure 
%# the systemic impact of this ratchet effect, the U.S. housing market is 
%# simulated with and without equity extractions, and the losses absorbed by 
%# mortgage lenders is estimated by valuing the embedded put option in 
%# non-recourse mortgages. The proposed systemic risk indicator for the housing 
%# market is the dollar-delta of this embedded put option. 
%#
%# # Input Description
%#
%# ## Input arg: begin_time
%# The vintage of the house.  1 <= begin_time <= t <= length(hpi)
%# ### Type: integer
%# ### Range: {1,...,+inf}
%# ### Dimensions: scalar
%#
%# ## Input arg: t
%# The time at which we want to calculate the value of the house. 
%# This must be > 1 as it used as an index. 1 <= begin_time <= t <=length(hpi)
%# ### Type: integer
%# ### Range: {1,...,+inf}
%# ### Dimensions: scalar
%#
%# ## Input arg: hpi
%# The Home Price Index monthly timeseries indicates increases in the value of 
%# existing homes.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: Tx1 matrix
%#
%# 1.  Rows represent Months (ascending).
%#
%#
%# ## Input arg: nhp
%# Monthly timeseries showing new house prices that is used for determining home
%# value at time of first sale.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: Tx1 matrix
%#
%# 1.  Rows represent Months (ascending).
%#
%#
%# # Output Description
%#
%# ## Output arg: v
%# House value at time t after adjusting for changes in HPI.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: scalar
%#
%# # Examples
%# NOTE: Numbers used in the examples are arbitrary valid values.  
%# They do not necessarily represent a realistic or plausible scenario.
%#
%#     begin_time = 1;
%#     t=5;
%#     hpi = [1; 1.015; 1.025; 1.035; 1.032; 1.05; 1.09];
%#     nhp = [1; 1.02; 1.03; 1.04; 1.05; 1.07; 1.11];
%#
%#     v = calc_value_house(begin_time, t, hpi, nhp);
%#
%# # References
%#
%# [Khandani, A. E., Lo, A. W., & Merton, R. C. (2012). Systemic risk and the 
%# refinancing ratchet effect. Journal of Financial Economics.]
%# (http://papers.ssrn.com/sol3/papers.cfm?abstract_id=1472892)
%#
%# [Bisias et al. (2012). A survey of systemic risk analytics 
%# (Working paper #0001). Washington, DC: Office of Financial Research, 89-94.]
%# (http://www.treasury.gov/initiatives/wsr/ofr/Documents/
%# OFRwp0001_BisiasFloodLoValavanis_ASurveyOfSystemicRiskAnalytics.pdf)
%#
%# # Code

% Run warning message
warning('OFRwp0001:UntestedCode', ...
    ['This version of the source code is very preliminary, ' ...
     'and has not been thoroughly tested. Users should not rely on ' ...
     'these calculations.']);





v = nhp(begin_time)*hpi(t)/hpi(begin_time);