function val = cca_fun(equity, volatility, risk_free_rate, default_barrier, ...
time_to_maturity, Init_asset_price, Init_asset_vol)
% 
%#
%# # License
%#
%# =============================================================================
%#
%# Copyright 2011, Dimitrios Bisias, Andrew W. Lo, and Stavros Valavanis
%#
%# COPYRIGHT STATUS:  This work was funded in whole or in part by the Office of 
%# Financial Research under U.S. Government contract TOSOFR-11-C-0001, and is, 
%# therefore, subject to the following license: The Government is granted for 
%# itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
%# worldwide license to reproduce, prepare derivative works,
%# distribute copies to the public, perform and display the work.  
%# All other rights are reserved by the copyright owner.
%# 
%# THIS SOFTWARE IS PROVIDED "AS IS".  YOU ARE USING THIS SOFTWARE AT YOUR OWN
%# RISK.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
%# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS, OR
%# THE UNITED STATES GOVERNMENT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
%# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
%# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
%# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%# 
%# =============================================================================
%#
%# # Synopsis
%#
%# cca_fun - Helper function to solve the system of non-linear equations in CCA
%#			
%# # Introduction 
%# NOTE: PART OF A SET OF 2 RELATED FILES: 
%#
%#  * cca.m
%#  * cca_fun.m
%#
%# Gray and Jobst (2010) propose using contingent claims analysis (CCA) to 
%# measure systemic risk from market-implied expected losses, with immediate 
%# practical applications to the analysis of implicit government contingent 
%# liabilities, i.e., guarantees. In addition, the framework also helps quantify 
%# the individual contributions of financial institutions to overall contingent
%# liabilities in the event of a systemic distress. Based on a sample of the 36 
%# largest financial institutions (banks, insurance companies, and asset 
%# managers), this systemic risk measurement framework generates an estimate of 
%# the joint contingent liabilities from market-implied government support. This 
%# approach does not only quantify the magnitude of potential risk transfer to 
%# the government but also helps indicate the contribution of individual 
%# institutions to contingent liabilities over time.
%#
%# # Input Description
%#
%# ## Input arg: equity
%# The market value of the equity of the firm.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: scalar
%#
%# ## Input arg: volatility
%# The volatility of the equity.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: scalar
%#
%# ## Input arg: risk_free_rate
%# The risk free rate in annualized terms.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: scalar
%#
%# ## Input arg: default_barrier
%# Face value of the outstanding debt at maturity.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: scalar
%#
%# ## Input arg: time_to_maturity
%# Time to maturity of the debt.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: scalar
%#
%# ## Input arg: cds_spread
%# The CDS spread daily time series for the firm. 
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: scalar
%#
%# # Output Description
%#
%# ## Output arg: a matrix containing expressions
%# This is a helper function to be solved implicitly. It contains
%# expressions derived from Black-Scholes intended to solve for asset_price 
%# and asset_vol.
%# ### Type: expression
%# ### Range: N/A
%# ### Dimensions: 2x1
%# 
%# # Examples
%# NOTE: Numbers used in the examples are arbitrary valid values.  
%# They do not necessarily represent a realistic or plausible scenario.
%#
%# # References
%#
%# [Gray, D., & Jobst, A. (2010, October). Systemic CCA-A Model Approach to 
%# Systemic Risk. In Deutsche Bundesbank/Technische Universitt Dresden 
%# Conference: Beyond the Financial Crisis: Systemic Risk, Spillovers and 
%# Regulation, Dresden.] (http://citeseerx.ist.psu.edu/viewdoc/download?
%# doi=10.1.1.186.9482&rep=rep1&type=pdf)
%#
%# [Bisias et al. (2012). A survey of systemic risk analytics 
%# (Working paper #0001). Washington, DC: Office of Financial Research, 77-80.]
%# (http://www.treasury.gov/initiatives/wsr/ofr/Documents/
%# OFRwp0001_BisiasFloodLoValavanis_ASurveyOfSystemicRiskAnalytics.pdf)
%#
%# # Code

% Run warning message
warning('OFRwp0001:UntestedCode', ...
    ['This version of the source code is very preliminary, ' ...
     'and has not been thoroughly tested. Users should not rely on ' ...
     'these calculations.']);


%define D1 and D2
d1 = (log(Init_asset_price^2/default_barrier)+(risk_free_rate+...
         (Init_asset_vol^4)/2)*time_to_maturity)/(Init_asset_vol^2*...
         sqrt(time_to_maturity));
d2 = d1 - Init_asset_vol^2*sqrt(time_to_maturity);

%define expressions to solve for Asset Price and Asset Vol
val = [...
       (equity - Init_asset_price^2*normcdf(d1)+default_barrier*exp(...
        -risk_free_rate*time_to_maturity)*normcdf(d2));
       ( equity*volatility - Init_asset_price^2*Init_asset_vol^2*normcdf(d1))...
      ];

