function ses = systemic_expected_shortfall(mes_training_sample, ...
lvg_training_sample, ses_training_sample, mes_firm, lvg_firm)
% 
%#
%# # License
%#
%# =============================================================================
%#
%# Copyright 2011, Dimitrios Bisias, Andrew W. Lo, and Stavros Valavanis
%#
%# COPYRIGHT STATUS:  This work was funded in whole or in part by the Office of 
%# Financial Research under U.S. Government contract TOSOFR-11-C-0001, and is, 
%# therefore, subject to the following license: The Government is granted for 
%# itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
%# worldwide license to reproduce, prepare derivative works,
%# distribute copies to the public, perform and display the work.  
%# All other rights are reserved by the copyright owner.
%# 
%# THIS SOFTWARE IS PROVIDED "AS IS".  YOU ARE USING THIS SOFTWARE AT YOUR OWN
%# RISK.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
%# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS, OR
%# THE UNITED STATES GOVERNMENT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
%# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
%# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
%# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%# 
%# =============================================================================
%#
%# # Synopsis
%#	
%# systemic_expected_shortfall - Calculates the systemic expected shortfall 
%# for a firm.
%#		
%# # Introduction 
%# NOTE: PART OF A SET OF 3 RELATED FILES: 
%#
%#  * systemic_expected_shortfall.m
%#  * marginal_expected_shortfall.m
%#  * leverage.m (auxiliary function)
%#
%# Acharya, Pedersen, Philippon, and Richardson (2010) argue that each financial 
%# institutions contribution to systemic risk can be measured as its systemic 
%# expected shortfall (SES), i.e., its propensity to be undercapitalized when 
%# the system as a whole is undercapitalized. SES is a theoretical construct and 
%# the authors use the following 3 measures to proxy it:
%# 
%# 1. The outcome of stress tests performed by regulators. The SES metric of a 
%# firm here is defined as the recommended capital that it was required to raise 
%# as a result of the stress test in February 2009.
%# 2. The decline in equity valuations of large financial firms during the 
%# crisis, as measured by their cumulative equity return from July 2007 to 
%# December 2008.
%# 3. The widening of the credit default swap spreads of large financial firms 
%# as measured by their cumulative CDS spread increases from July 2007 to 
%# December 2008.  
%# 
%# Given these proxies, the authors seek to develop leading indicators which 
%# predict an institutions SES; these leading indicators are marginal 
%# expected shortfall (MES) and leverage (LVG). 
%#
%# # Input Description
%#
%# ## Input arg: mes_training_sample
%# MES or value per firm defined as avg equity return during 5% worst days for 
%# overall market during training period.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: Fx1 matrix
%#
%# 1.  Rows represent firms
%#
%#
%# ## Input arg: lvg_training_sample
%# Leverage per firm defined on the last day of the period of training data. 
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: Fx1 matrix
%#
%# 1.  Rows represent firms
%#
%#
%# ## Input arg: ses_training_sample
%# Cumulative return per firm for date range after mes/lvg_training_sample.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: Fx1 matrix
%#
%# 1.  Rows represent firms
%#
%#
%# ## Input arg: mes_firm
%# The current firm MES used to calculate the firm SES value.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: scalar
%#
%#
%# ## Input arg: lvg_firm
%# The current firm leverage used to calculate the firm SES value. 
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: scalar
%#
%# # Output Description
%#     
%# ## Output arg: ses
%# The systemic risk that firm i poses to the system at a future time t.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: scalar
%#
%# # Examples
%# NOTE: Numbers used in the examples are arbitrary valid values.  
%# They do not necessarily represent a realistic or plausible scenario.
%#
%#     mes_training_sample = [ -.023, -.07, .01 ]';
%#     lvg_training_sample = [  1.8, 1.5, 2.2 ]';
%#     ses_training_sample = [ .3, .4, -.2 ]';
%#     mes_firm = .04;
%#     lvg_firm = 1.7;
%# 
%#     ses = systemic_expected_shortfall(mes_training_sample, ...
%#     lvg_training_sample, ses_training_sample, mes_firm, lvg_firm);
%#
%# # References
%#
%# [Acharya, Pedersen, Philippon, & Richardson. (2010). Measuring systemic 
%# risk.] (http://papers.ssrn.com/sol3/papers.cfm?abstract_id=2013815)
%#
%# [Bisias et al. (2012). A survey of systemic risk analytics
%# (Working paper #0001). 
%# Washington, DC: Office of Financial Research, 114-116.]
%# (http://www.treasury.gov/initiatives/wsr/ofr/Documents/
%# OFRwp0001_BisiasFloodLoValavanis_ASurveyOfSystemicRiskAnalytics.pdf)
%#
%# # Code

% Run warning message
warning('OFRwp0001:UntestedCode', ...
    ['This version of the source code is very preliminary, ' ...
     'and has not been thoroughly tested. Users should not rely on ' ...
     'these calculations.']);


num_firms = length(mes_training_sample);

% Regressors
X = [ones(num_firms,1) mes_training_sample lvg_training_sample];

% Regress systemic expected shortfall on marginal expected shortfall and
% leverage
betas = regress(ses_training_sample, X);

b = betas(2);
c = betas(3);
ses = (b*mes_firm + c*lvg_firm)/(b+c);
